#include "avrcore.h"
#include "decoder.h"

int datail;
int loop_max = 200;
char *rom_file = "rom_data.mem";

void usage() {
	fprintf(stderr, "usage avr8l_trace [-l loops] [-f rom_file]\n");
	exit(1);
}

uint8_t sreg_prev;
uint16_t PC_prev2;
uint16_t PC_prev;

int main(int argc, char *argv[]) {
	int i;
	int next = 0;
	FILE *fp;
	decode_init_lookup_table();

	for (i=1; i< argc; i++) {
		if (next == 1) 	{
			loop_max = atoi(argv[i]);
			next = 0;
		} else if (next == 2) {
			rom_file = argv[i];
			next = 0;
		} else if (!strcmp(argv[i], "-l")) {
			next = 1;
		} else if (!strcmp(argv[i], "-f")) {
			next = 2;
		} else if (!strcmp(argv[i], "-d")) {
			datail = 1;
		} else {
			usage();
		}
	}

	fp = fopen(rom_file, "r");
	if (!fp) usage();

	for (i=0; i< 8192; i++) {
		flash[i] = 0xffff;
	}
	for (i=0; i< 8192; i++) {
		uint32_t dat;
		if (fscanf(fp, "%04x", &dat) != 1) break;
		flash[i] = dat;
	}
	if (i < 2048) i = 2048;
	else if (i < 4096) i = 4096;
	else i = 8192;
	flash_size = i;

	log_done = 0;
	for (i=0; i< loop_max; i++) {
		struct opcode_info *opi;
		uint16_t opcode;
//printf("PC %02x INST %04x\n" ,PC, opcode);
		opcode = flash[PC];
		opi = decode_opcode(opcode);
		opi->func (NULL, opcode, opi->arg1, opi->arg2);
		if (sleep_occ) {
			printf("SLEEP  : PC %04x FLAGS %02x SP %04x\n"
				, PC_prev, sreg.reg, SP);
			break;
		}
		if (datail && !log_done && ( (sreg_prev != sreg.reg)
				    || (PC_prev != PC_prev2 + 1 )) ) {
			printf("       : PC %04x FLAGS %02x SP %04x\n"
				, PC_prev, sreg.reg, SP);
		}
		sreg_prev = sreg.reg;
		PC_prev2 = PC_prev;
		PC_prev = PC;
		log_done = 0;
	}
	return 0;
}
