#if !defined(__linux__) && !defined(_WIN32)

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <util/delay.h>
#include <string.h>
#include <stdio.h>

#else

#include <stdio.h>
#include <malloc.h>
#include <errno.h>
#include <fcntl.h>

#if !defined(__linux__) 
typedef char  int8_t;
typedef short int16_t;
typedef long  int32_t;
#endif

typedef unsigned char  uint8_t;
typedef unsigned short uint16_t;
typedef unsigned long  uint32_t;

#endif


#if defined(__linux__) || defined(_WIN32)
void dump_state(int16_t *state) {
	int8_t i;
	for (i=0; i< 8; i++) {
		printf("%04x",(uint16_t)(state[i]));
	}
	printf("\n");
}
#else
 #define dump_state()
#endif

uint16_t count[8];

void shuffle_init(int16_t *state) {
	state[0] = 0x3942;
	state[1] = 0xf681;
	state[2] = 0x6122;
	state[3] = 0x380c;
	state[4] = 0x2244;
	state[5] = 0xff97;
	state[6] = 0x23a7;
	state[7] = 0xa039;
}

void shuffle(int16_t *state) {
	int8_t i;
	int16_t out[8];

	state[8] = state[0];
//	dump_state(state);
	for (i=0; i< 8; i++,state++) {
	  uint8_t b,c;
	  b = (*(state + 1) >> 8) & 0xff;
	  c = (*(state + 1) >> 4) & 0x7;
	  count[c] ++;
	  switch (c) {
	  case 0:
		out[i] = *state * b;
		break;
	  case 1:
		out[i] = *state / (i+1);
		break;
	  case 2:
		out[i] = (*state << (i+2)) | ((uint16_t)(*state) >> (14-i));
		break;
	  case 3:
		out[i] = *state + 0x2562;
		break;
	  case 4:
		out[i] = (*state % (16-i)) << 6;
		break;
	  case 5:
		out[i] = *state + *(state +1);
		break;
	  case 6:
		out[i] = (*state / (16-i)) * b;
		break;
	  case 7:
		out[i] = *state * 3;
		break;
	  }
	  out[i] += 0x2562;
	}
//	dump_state(out);
	state -= 8;
	for (i=0; i< 8; i++) {
		state[i] = state[i] ^ out[i]; 
	}
//	dump_state(state);
}

#if defined(__linux__) || defined(_WIN32)
int main() {
	int8_t i;
	int16_t state[9];

	shuffle_init(state);

	for (i=0; i< 100; i++) { 
		shuffle(state);
	}
	/* 91a970f2f7442a4a4dca2e464b62dd4c */
	dump_state(state);
	for (i=0; i< 8; i++) {
		printf("%4d ",count[i]);
	}
	printf("\n");
	return 0;
}
#endif
